#!/usr/bin/env python

import os
import sys
import json


class ReportSummary:
    """
    Merge some json files into one summary json file.

    Methods:
        __init__(self, report_list:list, output_location=None,overwrite=False):
            Initialize the ReportSummary instance.

        _check_output_path(self):
            Check if the output information can be used to store json results.
            Check if the output path exists and if it can be overwritten.

        _check_input_files(self, source):
            Check if the input file is available to be parsed.

        _read_json_input(self, report):
            Read the contents of the json input file and convert it to a list.

        export_summary(self):
            Export the summarized results in json format.
    """

    def __init__(
        self,
        report_list: list,
        output_location=None,
        overwrite=False,
    ):
        """
        Initialize the ReportSummary instance.

        Args:
            report_list (list):
                List of json files to be summarized.
            output_location (str, optional):
                Path to the output location. Defaults to None.
            overwrite (bool, optional):
                If True, allows overwriting the output file. Defaults to False.
        """
        self._report_list = report_list
        self._output_location = output_location
        self._overwrite = overwrite
        self._default_output_name = "tooldistillator_summary.json"
        self._default_location = (
            f"{self._output_location}/{self._default_output_name}"
        )

    def _check_output_path(self):
        """
        Check if the output information can be used to store json results.
        Check if the output path exists and if it can be overwritten.
        Raises:
            ValueError:
                The output file exists, and the option
                --force to overwrite was not provided.
            FileExistsError:
                The path cannot be created.

        Returns:
            _io.TextIOWrapper: Console output or opened file to store results.
        """
        if self._output_location:
            # appending if more than one report
            if not os.path.exists(self._output_location):
                out_fh = open(self._output_location, "w")
            elif os.path.isfile(self._output_location):
                if self._overwrite is True:
                    out_fh = open(self._output_location, "a")
                else:
                    raise ValueError(
                        f"{self._output_location} exist,\
                        choose --force to overwrite the file"
                    )
            elif os.path.isdir(self._output_location) and not os.path.exists(
                self._default_location
            ):
                out_fh = open(self._default_location, "w")
            else:
                raise FileExistsError(
                    f"No valid output path: {self._output_location}"
                )
        else:
            out_fh = sys.stdout
        return out_fh

    def _check_input_files(self, source):
        """
        Check if the input is available to be parsed.

        Check if the file exists, if it's empty, or if the path does not exist.

        Args:
            source (str): Path to the input file.
        """
        try:
            if os.stat(source).st_size > 0:
                self.stream = open(source, "r")
        except (OSError, FileNotFoundError):
            print(f"{source} is empty or not found")
            sys.exit()

    def _read_json_input(self, report):
        """
        Read the json input file.

        Args:
            report (_io.TextIOWrapper): Opened file stream.

        Returns:
            list: Json data imported as a list.
        """
        validated_file = self._check_input_files(report)
        if validated_file != 0:
            json_file = json.loads(self.stream.read())
            self.stream.close()
            json_file = json_file[0]
            return json_file

    def export_summary(self):
        """
        Export the summarized results in json format.
        """
        output_destination = self._check_output_path()
        self.summary_json_list = [
            self._read_json_input(report) for report in self._report_list
        ]
        json.dump(
            self.summary_json_list,
            output_destination,
        )
