#!/usr/bin/env python

from Bio import SeqIO
import pandas as pd
import numpy as np
import re
from tooldistillator.interfaces import (
    ToolDistillatorResultIterator,
    logger
)

required_metadata = {
    "analysis_software_version": "amrfinderplus version number",
    "hid": "historic ID for amrfinderplus file from galaxy",
    "reference_database_version": "DB version",
    "point_mutation_report_path":
        "point mutation report file",
    "point_mutation_report_hid":
        "historic ID for point mutation report file from galaxy",
    "nucleotide_sequence_path":
        "nucleotide identified sequence fasta file",
    "nucleotide_sequence_hid":
        "historic ID for nucleotide sequence fasta file from galaxy",
}


class AmrFinderPlusIterator(ToolDistillatorResultIterator):
    """
    AMRFinderPlus tool parser

    | Filter results from amrfinderplus report file
    | Could also parse results for point mutation report file
    | and nucleotide identified sequence fasta file

    | Args:
    |     ToolDistillatorResultIterator: abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "amrfinderplus"
        super().__init__(source, metadata)
        self.tooldistillator_results["tool_has_database"] = True
        self.parse()

    def parse(self):
        """
        Extract information from AMRFinderPlus files
        """
        # skip any manually specified fields for later
        if self.stream:
            logger.info("add AMRFinderPlus report file")
            reader = pd.read_table(self.stream, header=0, sep="\t")
            reader = reader.replace({np.nan: None})
            reader = reader.to_dict(orient="records")
            self.tooldistillator_results["results"][0]["content"] = reader
        if self.metadata["point_mutation_report_path"]:
            logger.info("add AMRFinderPlus point mutation report file")
            self._check_input_files(
                self.metadata["point_mutation_report_path"]
                )
            reader = pd.read_table(self.stream, header=0, sep="\t")
            reader = reader.to_dict(orient="records")
            self.tooldistillator_results["results"].append(
                {
                    "name": "point_mutation_report",
                    "file_path": self.tooldistillator_results.pop(
                        "point_mutation_report_path"
                    ),
                    "hid": self.tooldistillator_results.pop(
                        "point_mutation_report_hid"),
                    "galaxy_id": "",
                    "content": reader,
                }
            )

        if self.metadata["nucleotide_sequence_path"]:
            logger.info("add AMRFinderPlus nucleotide sequence fasta file")
            self._check_input_files(
                self.metadata["nucleotide_sequence_path"]
                )
            parsed_fasta = self.__get_fasta_infos__()
            self.tooldistillator_results["results"].append(
                {
                    "name": "nucleotide_sequence",
                    "file_path": self.tooldistillator_results.pop(
                        "nucleotide_sequence_path"
                    ),
                    "hid": self.tooldistillator_results.pop(
                        "nucleotide_sequence_hid"),
                    "galaxy_id": "",
                    "content": parsed_fasta,
                }
            )

    def __get_fasta_infos__(self):
        """
        Extract fasta informations

        | Extract name, sequences, positions and information

        | Returns:
        |     dict: sequence informations
        """
        fasta_sequences = SeqIO.parse(self.stream, format="fasta")
        parsed_fasta_sequence = []
        for seqname in fasta_sequences:
            seq_results = {}
            seq_results["Name"] = seqname.id
            # Get information from sequence description
            p = "^([0-9A-Za-z_^ ]+):([0-9]+)-([0-9]+) strand:(.) ([^ ]+) (.*)$"
            match = re.match(p, seqname.description)
            seq_results["Contig id"] = match.group(1)
            seq_results["Start"] = match.group(2)
            seq_results["Stop"] = match.group(3)
            seq_results["Strand"] = match.group(4)
            seq_results["Gene symbol"] = match.group(5)
            seq_results["Sequence name"] = match.group(6)
            # Get more information from sequence
            seq_results["Length"] = len(seqname.seq)
            seq_results["Sequence"] = seqname.seq.__str__()
            parsed_fasta_sequence.append(seq_results)
        return parsed_fasta_sequence
