#!/usr/bin/env python

import re
import pandas as pd
from io import StringIO
from tooldistillator.interfaces import (
    ToolDistillatorResultIterator,
    logger
)

required_metadata = {
    "analysis_software_version": "fastqc version number",
    "hid": "historic ID for fastqc file from galaxy",
    "reference_database_version": "DB version",
    "html_report_path": "html fastqc report path",
    "html_report_hid": "historic ID for fastqc html report",
}


class FastqcIterator(ToolDistillatorResultIterator):
    """
    FastQC tool parser

    | Extract information from fastqc txt report

    | Args:
    |     ToolDistillatorResultIterator: abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "fastqc"
        self.metadata = metadata
        super().__init__(source, self.metadata)
        self.tooldistillator_results["tool_has_database"] = False
        self.parse()

    def parse(self):
        """
        Extract information from FastQC txt file
        """
        if self.stream:
            logger.info("Add fastqc report file")
            with open(self.stream.name, 'r') as file:
                data = file.read().split("\n", 1)[1]
            # Split the file content into modules
            mdls = [m.strip() for m in data.split(">>END_MODULE") if m.strip()]
            # Extract information from each module
            dc_df = {}
            for mdl in mdls:
                mdl = re.sub('#|>>', '', mdl)
                mdl_data = StringIO(mdl)
                try:
                    # Extract module name and convert to DataFrame
                    df_name = mdl.split("\t", 1)[0].replace(" ", "_").lower()
                    df = pd.read_csv(mdl_data, sep='\t', skiprows=1)
                    # Convert DataFrame to dictionary of records
                    dc_df[df_name] = df.to_dict(orient='records')
                except pd.errors.EmptyDataError:
                    logger.info(f'No informations in module: {mdl}')
            self.tooldistillator_results["results"][0]["content"].append(dc_df)
        self.extract_metadata_information(self.metadata)
