#!/usr/bin/env python

from Bio import SeqIO
import pandas as pd
from tooldistillator.interfaces import (
    ToolDistillatorResultIterator,
    logger
)

required_metadata = {
    "hid": "Historic ID to flye contigs file from Galaxy",
    "analysis_software_version": "flye version number",
    "reference_database_version": "DB version",
    "contig_graph_path": "Assembly graph file",
    "contig_graph_hid": "Historic ID to assembly graph from Galaxy",
    "tsv_file_path": "Assembly info file from flye",
    "tsv_file_hid": "Historic ID to assembly info file from Galaxy",
}


class FlyeIterator(ToolDistillatorResultIterator):
    """
    Flye tool parser

    | Filter results from contig fasta file. Could also keep path for:
    |     Alignment info file (tsv format)
    |     Contig graph file (gfa format)

    | Args:
    |     ToolDistillatorResultIterator: abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "flye"
        self.metadata = metadata
        super().__init__(source, metadata)
        self.tooldistillator_results["tool_has_database"] = False
        self.parse()

    def parse(self):
        """
        Assembly flye parser

        | Read contig assembly file to extract informations and sequences
        | Could also add path information for TSV file and GFA file
        """
        if self.stream:
            parsed_fasta = self.__get_fasta_infos__()
            self.tooldistillator_results["results"][0][
                "content"] = parsed_fasta
        if self.metadata["tsv_file_path"]:
            logger.info("add flye assembly tsv_file_path")
            self._check_input_files(self.metadata["tsv_file_path"])
            reader = pd.read_table(self.stream, sep="\t")
            reader = reader.to_dict(orient="records")
            self.tooldistillator_results["results"].append(
                {
                    "name": "assembly_infos",
                    "file_path": self.tooldistillator_results.pop(
                        "tsv_file_path"
                    ),
                    "hid": self.tooldistillator_results.pop(
                        "tsv_file_hid"),
                    "galaxy_id": "",
                    "content": reader,
                }
            )
        if self.metadata["contig_graph_path"]:
            logger.info("add flye assembly contig_graph_path")
            self._check_input_files(self.metadata["contig_graph_path"])
            self.tooldistillator_results["results"].append(
                {
                    "name": "assembly_graph",
                    "file_path": self.tooldistillator_results.pop(
                        "contig_graph_path"
                    ),
                    "hid": self.tooldistillator_results.pop(
                        "contig_graph_hid"),
                    "galaxy_id": "",
                    "content": [],
                }
            )

    def __get_fasta_infos__(self):
        """
        Extract fasta informations

        | Extract name, sequences, positions and IS information

        | Returns:
        |     dict: sequence informations
        """
        fasta_sequences = SeqIO.parse(self.stream, format="fasta")
        parsed_fasta_sequence = []
        for seqname in fasta_sequences:
            seq_results = {}
            seq_results["name"] = seqname.id
            seq_results["length"] = len(seqname.seq)
            seq_results["sequence"] = seqname.seq.__str__()
            parsed_fasta_sequence.append(seq_results)
        return parsed_fasta_sequence
